# -*- coding: utf-8 -*-
"""
collectr.api
------------

This module implements the public-facing portion of the collectr API.

:copyright: (c) 2013 Cory Benfield
:license: MIT License, see LICENSE for details.

"""
from .models import StaticDir


def update(directory, bucket, input_directory=''):
    """
    Updates an S3 bucket with the contents of a directory tree. Mirrors the
    tree into the S3 bucket. Returns nothing.

    :param directory: the root of the directory tree.
    :param bucket: the name of the S3 bucket to use.
    """
    d = StaticDir(directory)
    d.input_directory = input_directory
    d.update(bucket)
    return



# -*- coding: utf-8 -*-
"""
collectr.exceptions
-------------------

This module contains any exceptions raised by collectr.

:copyright: (c) 2013 Cory Benfield
:license: MIT License, see LICENSE for details.

"""


class MinifierError(IOError):
    """
    An error encountered during execution of the minifier.
    """



# -*- coding: utf-8 -*-
"""
collectr
--------

collectr is static file management for S3, for human beings. See README.md for
more.

:copyright: (c) 2013 Cory Benfield
:license: MIT, see LICENSE for details.
"""

__title__ = 'collectr'
__version__ = '0.0.4'
__build__ = 0x000001
__author__ = 'Cory Benfield'
__license__ = 'MIT'
__copyright__ = 'Copyright 2013 Cory Benfield'

from .api import *
from . import exceptions



# -*- coding: utf-8 -*-
"""
collectr.utils
--------------

This module implements utility functions and holds utility data for collectr.

:copyright: (c) 2013 Cory Benfield
:license: MIT License, see LICENSE for details.

"""
import os

# Directories created by version control software.
VCS_DIRS = ['.git', '.svn', '.hg']


def tree_walk(directory):
    """
    Recursively walk a directory tree. Returns a list of files in the
    tree. Does not walk down folders created by version control.
    """
    files = []

    for dirpath, subdirs, filenames in os.walk(directory):
        for name in filenames:
            files.append(os.path.join(dirpath, name))

        # Ignore version control directories.
        for vcs_dir in VCS_DIRS:
            if vcs_dir in subdirs:
                subdirs.remove(vcs_dir)

    return files


def match_regexes(regexes, string):
    """
    Given a string, determine if it matches any of a list of regular
    expressions. Returns True if it does, False if it doesn't.
    """
    match = [regex.search(string) for regex in regexes if regex.search(string)]

    if match:
        return True
    else:
        return False


def move_path(current_dir, new_dir, path):
    """
    Move a path from one directory to another. Given the directory the path is
    currently in, moves it to be in the second. The path below the given
    directory level remains the same.
    """
    if current_dir[-1] != '/':
        current_dir += '/'
    if new_dir[-1] != '/':
        new_dir += '/'

    return path.replace(current_dir, new_dir, 1)


def minified_filename(path):
    """
    Given a path, returns a new path with the filename changed to a minified
    form, e.g. test.css -> test-min.css
    """
    name, extension = os.path.splitext(path)
    name += "-min"
    return name + extension


def get_extension(path):
    """
    Given a path, returns the extension, excluding the dot, e.g. test.css ->
    css.
    """
    _, extension = os.path.splitext(path)
    return extension[1:]


def default_minifier():
    """
    Determine the default minifier dict.
    """
    # TODO: Come back and generalise this.
    return 'yuicompressor -o {out_name} {in_name}'



#! /usr/bin/env python

import sys
import os
import re

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] in ('submit', 'publish'):
    os.system('python setup.py sdist upload')
    sys.exit()

__version__ = ''
with open('collectr/__init__.py', 'r') as fd:
    reg = re.compile(r'__version__ = [\'"]([^\'"]*)[\'"]')
    for line in fd:
        m = reg.match(line)
        if m:
            __version__ = m.group(1)
            break

packages = ['collectr']

setup(
    name='collectr',
    version=__version__,
    description='Static file management for everyone.',
    long_description=open('README.rst').read(),
    author='Cory Benfield',
    author_email='cory@lukasa.co.uk',
    url='http://www.lukasa.co.uk/',
    scripts=['scripts/collect_static'],
    packages=packages,
    package_data={'': ['LICENSE']},
    package_dir={'collectr': 'collectr'},
    include_package_data=True,
    install_requires=['boto'],
    license=open('LICENSE').read(),
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7'
        ),
    )



# -*- coding: utf-8 -*-
"""
collectr.models
---------------

This module contains the main models used by collectr.

:copyright: (c) 2013 Cory Benfield
:license: MIT License, see LICENSE for details.

"""
from .utils import (tree_walk, match_regexes, move_path, minified_filename,
                    get_extension, default_minifier)
from .exceptions import MinifierError
import re
import subprocess
from boto.s3.connection import S3Connection
from boto.exception import S3ResponseError
from boto.s3.key import Key


class StaticDir(object):
    """
    This class represents a single directory or directory tree of static files.
    This class can be created by the user, or used internally in collectr.

    :param directory: The root of the directory tree.
    """
    def __init__(self, directory):
        #: The root of the directory tree.
        self.directory = directory

        #: A string representing the location of static files that have yet to
        #: be minified. Files in this directory will be minified, and the
        #: minified versions will be saved to 'dir'.
        self.input_directory = None

        #: The minifier to use on CSS and/or Javascript files. May be a
        #: dictionary whose keys correspond to file extensions or a string. The
        #: string is considered a special case, and will be applied to both
        #: Javascript and CSS files.
        #: If the dictionary is used, the keys correspond to a command-line to
        #: run. This should be a python format string with two string
        #: variables, {in_name} and {out_name}. These refer to the input and
        #: output filename respectively. The string must be able to have
        #: .format() called on it.
        self.minifier = default_minifier()

        #: Whether to update all files, regardless of whether they have been
        #: changed.
        self.force_update = False

        #: Files to ignore. Should be a list of regular expressions. Everything
        #: that matches any of these regular expressions will be totally
        #: ignored. The regex will be applied to the relative path.
        self.ignore = []

        #: A dictionary of keys and values that correspond to the metadata that
        #: should be applied to the files. This metadata will be applied to
        #: _all_ the files found by this :class:`StaticDir <StaticDir>`.
        self.metadata = {}

    def update(self, bucket_name):
        """
        Connect to S3 and update the bucket with the static files from the
        directory.

        :param bucket_name: The name of the S3 bucket to upload to.
        """
        self.minify_files()
        files = self.enumerate_files(self.directory)
        conn = self.connect_s3()
        self.upload_files(files, bucket_name, conn)
        return

    def enumerate_files(self, directory):
        """
        Enumerate all the files beneath a directory. Walks into all
        directories except for those created by version control.

        :param directory: The root of the tree.
        """
        files = tree_walk(directory)

        # Ignore some files.
        files = self.filter_files(files)

        return files

    def filter_files(self, files):
        """
        Given a list of files, remove any that match any of a list of regular
        expressions.
        """
        tests = [re.compile(x) for x in self.ignore]

        # Bail early if we don't have any regexes to match.
        if tests:
            # Remove files that are matched by regexes.
            files = [name for name in files if not match_regexes(tests, name)]

        return files

    def minify_files(self):
        """
        Takes all the files either in the main directory or the input directory
        and minifies them. If the files came from the input directory, moves
        them to the main directory.
        """
        if self.input_directory:
            files = self.enumerate_files(self.input_directory)
        else:
            files = self.enumerate_files(self.directory)

        # Strings are a special case: apply that special case.
        if isinstance(self.minifier, basestring):
            minifier = {'css': self.minifier, 'js': self.minifier}
        else:
            minifier = self.minifier

        # For each file, if its extension has a minifier associated with it,
        # apply it.
        for name in files:
            try:
                command = minifier[get_extension(name)]
                command = command.format(in_name=name,
                                         out_name=self.get_output_name(name))

                rc = subprocess.call(command, shell=True)

                if rc != 0:
                    raise MinifierError("Error occurred during minification.")

            except KeyError:
                # If there's no minifier command, don't touch it.
                continue

        return

    def get_output_name(self, input_filename):
        """
        When minifying a file, determine its output filename. This depends on
        whether it's being copied to a new directory.
        """
        if self.input_directory:
            filename = move_path(self.input_directory,
                                 self.directory,
                                 input_filename)
            filename = minified_filename(filename)
        else:
            filename = minified_filename(input_filename)

        return filename

    def connect_s3(self):
        """
        Connect to S3. Returns the boto connection object.
        """
        return S3Connection()

    def upload_files(self, files, bucket_name, connection):
        """
        Given a list of files, an Amazon S3 bucket and a connection, uploads
        the files to the bucket. If the bucket doesn't exist, creates it.
        """
        # First get the bucket. If it exists, great. If not, create it.
        try:
            bucket = connection.get_bucket(bucket_name)
        except S3ResponseError:
            bucket = connection.create_bucket(bucket_name)

        # For each file, create an S3 key and upload the data. Then, set the
        # metadata.
        for name in files:
            key = Key(bucket)
            key.key = name.replace(self.directory, '', 1)
            key.set_contents_from_filename(name)

            for metakey, metavalue in self.metadata.iteritems():
                key.set_metadata(metakey, metavalue)

            # Set the visibility to public-read.
            key.set_acl('public-read')

        # All done.
        return
